package  rsf.operator;

import   jam.vector.*;
import	 rsf.vector.*;
import   sep.io.ArgumentKeeper;
/**
 * <BR><!--#include file="./source.inc"--><BR>
 * 
 * Implements elementwise division r = d/y where currently 
 * denominator y is handed in at construction time. I wonder if we want 
 * another operator which requires the nominator d at construction time. 
 */
public class ElwDiv implements jam.operator.Operator { 
  RsfSpace spce ; 
  Rsf      dnmt ; // denominator

  public ElwDiv(Rsf denominator) {
    this.dnmt = denominator; 
    this.spce = (RsfSpace) dnmt.getSpace();
  }

  // operator superclass methods

  public jam.JamObject copy  () { return new ElwDiv(dnmt); }

  public Vector image(Vector d) {
    Vector r = getRange().newMember();
    elwDiv(false,r,d,dnmt);
    return r;
  }
  /**
   * image(d,d) for in-place operation. (In general this is not 
   * a good idea but for elementwise operations such in-place 
   * operations are straightforward. 
   */
  public void image(Vector d, Vector r){
    elwDiv(false,r,d,dnmt);
  }
  public void addImage(Vector d, Vector r){
    // alternative to add flag I could build a dummy
    elwDiv(true,r,d,dnmt);
  } 
  public void residual(Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d,r); 
  }

  public  Space getDomain         () { return spce; }
  public  Space getRange          () { return spce; }

  /**
   * invocations: \br
   * ElwDiv < nominator.H  denom=denominator.H > rslt.H 
   */
  public static void main(String[] args) { 
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);
    String  dnmtName   = argKeeper.getString("denom","");
    Rsf nmtr  = (Rsf) RsfFactory.newRsf(System.in);
    Rsf dnmt  = (Rsf) RsfFactory.newRsf(dnmtName);
    Rsf rslt = (Rsf) dnmt.getSpace().newMember();
    elwDiv(false,rslt,nmtr,dnmt);
    rslt.write(System.out); 
  }

  public static void elwDiv(       Rsf rslt, Rsf nmtr, Rsf dnmt) {
    elwDiv(                 false,     rslt,     nmtr,     dnmt);  
  }

  public void elwDiv(boolean add, Vector rslt, Vector nmtr, Rsf dnmt) {
    elwDiv(                  add, (Rsf)  rslt, (Rsf)  nmtr,     dnmt); 
  }

  public static void elwDiv(boolean add, Rsf rslt, Rsf nmtr, Rsf dnmt) {
    int[] rs = ((RsfSpace) rslt.getSpace()).getNSamples();
    int[] nm = ((RsfSpace) nmtr.getSpace()).getNSamples();
    int[] dn = ((RsfSpace) dnmt.getSpace()).getNSamples();

    if ((! juice.util.JuiceUtil.equalArrays(rs, nm)) || 
        (! juice.util.JuiceUtil.equalArrays(nm, dn)) ){
      System.err.println(
	 "Error: elementwise operation requires equal array size!");
    }

    switch (rs.length) {
    case 1: 
      float[]   rslt1 = ((Rsf1) rslt).d;
      float[]   nmtr1 = ((Rsf1) nmtr).d;
      float[]   dnmt1 = ((Rsf1) dnmt).d;
      for (int i0=0; i0<rs[0]; i0++) {
	if (add)  
	  rslt1[i0]         += nmtr1[i0]         / dnmt1[i0];
	else 
	  rslt1[i0]          = nmtr1[i0]         / dnmt1[i0];
      }
      break;
    case 2: 
      float[][] rslt2 = ((Rsf2) rslt).d;
      float[][] nmtr2 = ((Rsf2) nmtr).d;
      float[][] dnmt2 = ((Rsf2) dnmt).d;
      for (int i1=0; i1<rs[1]; i1++) {
      for (int i0=0; i0<rs[0]; i0++) {
	if (add) 
	  rslt2[i1][i0]     += nmtr2[i1][i0]     / dnmt2[i1][i0];
	else
	  rslt2[i1][i0]      = nmtr2[i1][i0]     / dnmt2[i1][i0];
      }}
      break;
    case 3: 
      float[][][] rslt3 = ((Rsf3) rslt).d;
      float[][][] nmtr3 = ((Rsf3) nmtr).d;
      float[][][] dnmt3 = ((Rsf3) dnmt).d;
      for (int i2=0; i2<rs[2]; i2++) {
      for (int i1=0; i1<rs[1]; i1++) {
      for (int i0=0; i0<rs[0]; i0++) {
	if (add) 
	  rslt3[i2][i1][i0] += nmtr3[i2][i1][i0] / dnmt3[i2][i1][i0];
	else 
	  rslt3[i2][i1][i0]  = nmtr3[i2][i1][i0] / dnmt3[i2][i1][i0];
      }}}
      break;
    }
  }
}







