package   rsf.operator;

import    jam.vector.*;
import 	  rsf.vector.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 * 
 * Pointwise inverse: 
 * Takes an Rsf and computes <pre>1/x[i]<pre>.
 * If <pre>x[1]<pre> is close to zero, I replace it by <pre>eps<pre>.
 * 
 */
public class ElwInv implements jam.operator.Operator { 
  RsfSpace spc; 
  float    eps; 

  /** 
   * Any element <pre>e<pre> with <pre>Abs(e) < eps<pre> 
   * is replaced by <pre>eps<pre> or <pre>-eps<pre> 
   * as appropiate. This prevents a division by zero.  
   * A practical eps value might be 1.0e-12f.
   * 
   * No catching of NaN or INFINITE numbers yet 
   */
  public ElwInv(RsfSpace spc, float eps) {
    this.spc = spc;
    this.eps = eps; 
  }
  public ElwInv(RsfSpace spc, Float eps) {
    this(spc, eps.floatValue()); 
  }
  /** 
   * eps = java.lang.Float.MIN_VALUE
   */
  public ElwInv(RsfSpace spc) {
    this(spc, java.lang.Float.MIN_VALUE);
  }

  // operator superclass methods

  public jam.JamObject copy  () { return new ElwInv(spc,eps); }

  public Vector image(Vector d) {
    Vector r = getRange().newMember();
    r.copyFrom(d); 
    elwInv(r,eps);
    return r;
  }
  public void image(Vector d, Vector r){
    r.copyFrom(d);
    elwInv(r,eps);
  }
  public void addImage(Vector d, Vector r){ 
    Vector tmp = getDomain().newMember();
    tmp.copyFrom(d);
    elwInv(tmp,eps);
    r.add(tmp); 
  } 
  public void residual(Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d,r); 
  }

  public  Space getDomain         () { return spc; }
  public  Space getRange          () { return spc; }

  public static void main(String[] args) { 
    Rsf data = RsfFactory.newRsf(System.in);
    //inv(data,1.0e-12f);
    elwInv(data,java.lang.Float.MIN_VALUE);
    data.write(System.out); 
  }

  public static void elwInv(Vector data) {
    elwInv(data, java.lang.Float.MIN_VALUE); 
  }

  public static void elwInvDivByZeroIsZero(Vector data) {
    elwInv(data, java.lang.Float.MIN_VALUE, 0.f); 
  } 
  public static void elwInv(Vector data, float eps) {
    elwInv(data,eps,1.f/eps);
  }
  /**
   * In general this should not be invoked. Maybe I should make it private. 
   * A user may set eps and upper to nonsensical values.<BR>
   * @param: eps:  absolute value of the lower limit of floats inverted 
   *               correctly.  
   * @param upper: absolute value assigned to the inverse off input values 
   *               that are equal or smaller than eps.
   */
  public static void elwInv(Vector data, float eps, float upper) {
    if (eps == 0.f) upper = 0; 
    int[] nn = ((RsfSpace) data.getSpace()).getNSamples();
    switch (nn.length) {
    case 1: 
      float[] x1 = ((Rsf1) data).d;
      for (int i1=0; i1<nn[0]; i1++) {
	if (Math.abs(x1[i1]) <= eps) 
	  x1[i1] = (x1[i1] < 0.f)               ? - upper : upper;
	else 
	  x1[i1]  = 1.f/x1[i1];
      }
      break;
    case 2: 
      float[][] x2 = ((Rsf2) data).d;
      for (int i2=0; i2<nn[1]; i2++) {
      for (int i1=0; i1<nn[0]; i1++) {
	if (Math.abs(x2[i2][i1]) <= eps) 
	  x2[i2][i1] = (x2[i2][i1] < 0)         ? - upper : upper;
	else
	  x2[i2][i1]  = 1.f/x2[i2][i1];
      }}
      break;
    case 3: 
      float[][][] x3 = ((Rsf3) data).d;
      for (int i3=0; i3<nn[2]; i3++) {
      for (int i2=0; i2<nn[1]; i2++) {
      for (int i1=0; i1<nn[0]; i1++) {
	if (Math.abs(x3[i3][i2][i1]) <= eps) 
	  x3[i3][i2][i1] = (x3[i3][i2][i1] < 0) ? -upper : upper;
	else
	  x3[i3][i2][i1]  = 1.f/x3[i3][i2][i1];
      }}}
      break;
    }
  }
}







