package  rsf.operator;

import   jam.vector.*;
import	 rsf.vector.*;
import   sep.io.ArgumentKeeper;
/**
 * <BR><!--#include file="./source.inc"--><BR>
 * 
 * Elementwise multiplication of two Rsfs
 * (this should be linear) 
 */
public class ElwMul implements jam.operator.Operator { 
  RsfSpace spc ; 
  Rsf      fac ;

  public ElwMul(Rsf fac) {
    this.fac = fac; 
    this.spc = (RsfSpace) fac.getSpace();
  }

  // operator superclass methods

  public jam.JamObject copy  () { return new ElwMul(fac); }

  public Vector image(Vector d) {
    Vector r = getRange().newMember();
    elwMul(r,d,fac);
    return r;
  }
  /**
   * image(d,d) for in-place operation. (In general this is not 
   * a good idea but for elementwise operations such in-place 
   * operations are straightforward. 
   */
  public void image(Vector d, Vector r){
    elwMul(r,d,fac);
  }
  public void addImage(Vector d, Vector r){
    // alternative to building a tmp dummy, I could use add flag. 
    Vector tmp = getDomain().newMember();
    elwMul(tmp,d,fac);
    r.add(tmp); 
  } 
  public void residual(Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d,r); 
  }

  public  Space getDomain         () { return spc; }
  public  Space getRange          () { return spc; }

  /**
   * invocations: \br
   * ElwMul <    fac1.H  fac=fac2.H > rslt.H \br
   * ElwMul facx=fac1.H facy=fac2.H > rslt.H
   */
  public static void main(String[] args) { 
    ArgumentKeeper argKeeper = new ArgumentKeeper(args);
    Rsf facx, facy; 
    String  facName   = argKeeper.getString("fac","");
    if (facName.length() != 0) {
      facx = (Rsf) RsfFactory.newRsf(System.in);
      facy = (Rsf) RsfFactory.newRsf(facName);
    }
    else {
      String  facxName  = argKeeper.getString("facx");
      String  facyName  = argKeeper.getString("facy");
      facx = (Rsf) RsfFactory.newRsf(facxName);
      facy = (Rsf) RsfFactory.newRsf(facyName);
    }
    Rsf rslt = (Rsf) facx.getSpace().newMember();
    elwMul(rslt,facx,facy);
    rslt.write(System.out); 
  }

  public void elwMul(Vector rslt, Vector facx, Rsf facy) {
    elwMul((Rsf) rslt, (Rsf) facx, (Rsf) facy); 
  }

  public static void elwMul(Rsf rslt, Rsf facx, Rsf facy) {
    int[] rs = ((RsfSpace) rslt.getSpace()).getNSamples();
    int[] f1 = ((RsfSpace) facx.getSpace()).getNSamples();
    int[] f2 = ((RsfSpace) facy.getSpace()).getNSamples();

    if ((! juice.util.JuiceUtil.equalArrays(rs, f1)) || 
        (! juice.util.JuiceUtil.equalArrays(f1, f2)) ){
      System.err.println(
	 "Error: elementwise operation requires equal array size!");
    }

    switch (rs.length) {
    case 1: 
      float[]   rslt1 = ((Rsf1) rslt).d;
      float[]   facx1 = ((Rsf1) facx).d;
      float[]   facy1 = ((Rsf1) facy).d;
      for (int i0=0; i0<rs[0]; i0++) {
	rslt1[i0]          = facx1[i0]         * facy1[i0];
      }
      break;
    case 2: 
      float[][] rslt2 = ((Rsf2) rslt).d;
      float[][] facx2 = ((Rsf2) facx).d;
      float[][] facy2 = ((Rsf2) facy).d;
      for (int i1=0; i1<rs[1]; i1++) {
      for (int i0=0; i0<rs[0]; i0++) {
	rslt2[i1][i0]      = facx2[i1][i0]     * facy2[i1][i0];
      }}
      break;
    case 3: 
      float[][][] rslt3 = ((Rsf3) rslt).d;
      float[][][] facx3 = ((Rsf3) facx).d;
      float[][][] facy3 = ((Rsf3) facy).d;
      for (int i2=0; i2<rs[2]; i2++) {
      for (int i1=0; i1<rs[1]; i1++) {
      for (int i0=0; i0<rs[0]; i0++) {
	rslt3[i2][i1][i0]  = facx3[i2][i1][i0] * facy3[i2][i1][i0];
      }}}
      break;
    }
  }
}







