package  rsf.operator;

import   rsf.vector.*;
import   sep.operator.*;
import   sep.io.*;
import   jam.vector.*;

/** 
 * This class generates a synthetic Rsf. 
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * This is rather trivial. 
 */

public class Spike extends SepOperator {
  private RsfSpace domAndRange; 
  private Rsf      outputFilter = null; 
  private float    magnitude = 1.f; 
  private int[]    coords; 

  public Spike(RsfSpace domAndRange) {
    super(domAndRange, domAndRange); 
    this.domAndRange = domAndRange;
    coords = new int[domAndRange.getDimension()];
    for (int i=0; i<coords.length; i++)  coords[i] = 0; 

  }
  Spike(Spike src) {
    this(src.domAndRange); 
    outputFilter      = src.outputFilter;
    magnitude   = src.magnitude;
    coords      = new int[src.coords.length];
    for (int i=0; i< coords.length; i++) coords[i] = src.coords[i]; 
  }
  public jam.JamObject copy() { return new Spike(this); }

  public void setSpike(int[] coords) {
    if (this.coords.length != coords.length) 
      System.err.println("Error: dimensional mismatch coordinates and Space");
    for (int i=0; i<coords.length; i++)  this.coords[i] = coords[i]; 
  }
  public void setMagnitude(float magnitude) { this.magnitude = magnitude; }

  public void setOutputFilter(Rsf outputFilter) { 
    if (outputFilter.getDimension() != domAndRange.getDimension()) 
      System.err.println("Error: filter and range dimensional mismatch"); 
    this.outputFilter = outputFilter;
  }

  Rsf filterOutput(Rsf unfiltered) {
    return rsf.util.Convolver.convolve(outputFilter, unfiltered);
  }

  protected void apply( boolean add, Vector dom, Vector ran) {
    // should check dom.getSpace() equals ran.getSpace()
    if (!add) ran.zero();
    ((Rsf) ran).setDatum(coords, magnitude);
    if (outputFilter != null) 
      ran = filterOutput((Rsf) ran);
  }

  /**
   * Specify the output data set: 
   * @param <tt>nsamples=n0,n1,n2,..</tt> where ni are Strings that convert to 
   *   integers. For example <tt>nsamples=100,20,10</tt>.
   *   The number of integers defines the physical dimension. 
   *   The integer values specify the axis' number of samples.
   *   Default values for axis members delta, offset, unit,lable.
   * @param <tt>axis1=n,delta,offset,unit,label</tt> where 
   *   String n converts to integer, 
   *   String delta converts to float, 
   *   String offset converts to float, 
   *   String unit and lable stay Strings.
   *   For example <tt>axis1=101,0.04f,0.01f,time,sec</tt>.
   *
   *   The number of axis statements (axis1={..}, axis2={..}, .., axisN={..}) 
   *   specifies the physical dimensioin. 
   *
   * Specify the spike location and magnitude: 
   * @param spike1=n0,n1,n2
   *   for example spike1=2,3,4,55.5f specifies the spike coordinates 
   *   as i0=2, i1=3, i3=4 and amplitude as 55.5f. The number of coordinates
   *   have to agree with space dimensions. 
   * @param int[] spike2=int n3,int n2,int n1
   *   Same as above but the amplitude is defaulted to 1.f.
   * @param mag1=magnitude specifies the magnitude. Default magnitude=1. 
   * 
   * Several spike and mag pairs can be set with increasing numbering: 
   * spike1={..} spike2={..} ... k1=amp1 k2=amp2 ... 
   */
  public static void main(String[] args) {
    Rsf data = null;
    ArgumentKeeper cmd = new ArgumentKeeper(args);
    // Specify the output volume: 
    if (cmd.containsKey("nsamples")) {
      data = RsfFactory.newRsf(cmd.getIntArray("nsamples"));
    }
    else if (cmd.containsKey("axis1")) {
      int dim = 0;
      for (int i=1; cmd.containsKey(new String("axis" + i)); i++) dim++; 
      Axis[] ax = new Axis[dim];
      for (int i=0; i<dim; i++) 
	ax[i] = cmd.getAxis(new String("axis" + 1));
      data = RsfFactory.newRsf(ax); 
    }
    else {
      System.err.println("Error: insufficient input information");
    }

    // Specify the Spikes: 
    Spike generator = new Spike((RsfSpace) data.getSpace());
    for (int i=1; cmd.containsKey(new String("spike" + i)); i++) {
      int[] spike = cmd.getIntArray(new String("spike" + i));
      if ( spike.length != data.getDimension() )
	System.err.println("Error: dimensional mismatch");
      float mag   = cmd.getFloat(new String("mag" + i), "1.f");
      generator.setSpike(spike);
      generator.setMagnitude(mag);
      generator.addImage(data,data); 
    }
    if (cmd.containsKey("out"))
      data.write(cmd.getString("out"));
    else 
      data.write(System.out); 
  }
}




