package  rsf.operator;

import   jam.vector.*;
import juice.vector.*;
import 	 rsf.vector.*;
import   sep.io.ArgumentKeeper;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * Computes the variance of an input Rsf: Sum(x_i^2 - \bar{a}^2) \br 
 * All the elements of the output Rsf are constant value. 
 * This is not a SepOperator since for convenience apply is a static method. 
 */
public class StatVar implements jam.operator.Operator {
  RsfSpace domain;
  RsfSpace range;

  public StatVar(Space domain, Space range) {
    this.domain = (RsfSpace) domain;
    this.range  = (RsfSpace) range; 
  }

  /**
   * This constructor is superceeded by StatVarFactory (here only for demo)
   */
  public StatVar(Space spc, String spcName) {
    if (spcName.equals("domain")) {
      this.domain = (RsfSpace) spc; 
      this.range  = (RsfSpace) createRange((RsfSpace) spc); 
    }
    else {
      this.domain = (RsfSpace) createDomain((RsfSpace) spc); 
      this.range  = (RsfSpace) spc;
    } 
  }
  static Space createDomain(Space range)  { 
    return (Space) ((RsfSpace) range).copy() ; 
  }
  static Space createRange( Space domain) { 
    return (Space) ((RsfSpace) domain).copy(); 
  }

  // operator superclass methods

  public jam.JamObject copy  () { return new StatVar(domain, range); }

  public Vector image(Vector d) {
    Vector r = getRange().newMember();
    statVar(false,r,d);
    return r;
  }
  public void image(Vector d, Vector r){
    statVar(false,r,d);
  }
  /** 
   * I wonder if this method makes any sense for Var(). I 
   * throw an error instead for now. 
   */
  public void addImage(Vector d, Vector r){ 
    statVar(true,r,d);
  } 
  public void residual(Vector d, Vector b, Vector r) {
    r.neg(b);
    addImage(d,r); 
  }

  public  Space getDomain         () { return domain; }
  public  Space getRange          () { return range ; }

  /**
   * Invocation: 
   * StatVar < i.H > o.H
   */
  public static void main(String[] args) { 
    Rsf data = RsfFactory.newRsf(System.in);
    data.fill(StatVar.statVar(data)); 
    data.write(System.out); 
  }

  public static void statVar(boolean add, Vector r, Vector d) {
    if (add) System.err.println("add flag not implemented!"); 
    if (d.getClass().getName().equals("RealVec")) 
      ((RealVec)r).setValue(statVar((Rsf) d));
    else 
      ((Rsf)    r).fill    (statVar((Rsf) d));
  }

  public static float statVar(Rsf d) {
    int[] dd = ((RsfSpace) d.getSpace()).getNSamples();
    float var   = 0.f; 
    
    switch (dd.length) {
    case 1: 
      float[] d0 = ((Rsf1) d).d;
      for (int i0=0; i0<dd[0]; i0++) {
	var += d0[i0]         * d0[i0];
      }
      break;
    case 2: 
      float[][] d1 = ((Rsf2) d).d;
      for (int i1=0; i1<dd[1]; i1++) {
      for (int i0=0; i0<dd[0]; i0++) {
	var += d1[i1][i0]     * d1[i1][i0];
      }}
      break;
    case 3:
      float[][][] d2 = ((Rsf3) d).d;
      for (int i2=0; i2<dd[2]; i2++) {
      for (int i1=0; i1<dd[1]; i1++) {
      for (int i0=0; i0<dd[0]; i0++) {
	var += d2[i2][i1][i0] * d2[i2][i1][i0]; 
      }}}
      break;
    }
    var = var / d.size();
    float mean = StatMean.statMean(d); 
    var = var - mean*mean; 
    var = (float) Math.sqrt(var); 
    return var;
  }
}







