package   rsf.solver;

import    sep.operator.SepSelfAdjointOperator;
import    jam.operator.*;
import    jam.vector.*;
import    rsf.vector.*;
import    jam.JamObject;

/**
 * This probably doesn't belong in rsf.solver, 
 * but MissingDataEquations is the only
 * thing that currently uses it.  Also, the name is probably not so good.
 *
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class SepMask extends SepSelfAdjointOperator implements Mask {
  protected Rsf     msk;

  /**
   * Create a mask which zeroes the components which are zero in maskRsf.<BR>
   * @param maskRsf may be subsequently changed without affecting "this".
   */
  public SepMask(Rsf maskRsf) {
    this(maskRsf, false);
  }

  /**
   * If complement = false, this is the same as <TT>SepMask(Rsf maskRsf)</TT>
   * <BR>
   * If complement = true, the mask zeroes those components which are
   * non-zero in maskRsf.
   */
  public SepMask(Rsf maskRsf, boolean complement) {
    super(maskRsf.getSpace());
    if (maskRsf.getDimension() > 6) {
      System.err.println("Error in SepMask constructor: dimension of " +
			 "maskRsf greater than 6");
      return;
    }
    createMask(maskRsf, complement);
  }

  protected SepMask(SepMask src) {
    super(src);
    this.msk = (Rsf) src.msk.copy();
  }

  void createMask(Rsf rsf, boolean complement) {
    msk = (Rsf) rsf.getSpace().newMember();
    switch (rsf.getDimension()) {
    case 1: createMask1((Rsf1) rsf, complement); break;
    case 2: createMask2((Rsf2) rsf, complement); break;
    case 3: createMask3((Rsf3) rsf, complement); break;
    case 4: createMask4((Rsf4) rsf, complement); break;
    case 5: createMask5((Rsf5) rsf, complement); break;
    case 6: createMask6((Rsf6) rsf, complement); break;
    }
  }

  void createMask1(Rsf1 rsf, boolean complement) {
    float[] vals = rsf.d;
    float[] mskd = ((Rsf1) msk).d;
    if (complement)
      for (int i = 0; i < vals.length; i++)
	mskd[i] = (vals[i] != 0F) ? 0F:1F;
    else
      for (int i = 0; i < vals.length; i++)
	mskd[i] = (vals[i] == 0F) ? 0F:1F;
  }
  
  void createMask2(Rsf2 rsf, boolean complement) {
    int n0 = ((RsfSpace) rsf.getSpace()).getNSamples(0);
    int n1 = ((RsfSpace) rsf.getSpace()).getNSamples(1);
    float[][] mskd = ((Rsf2) msk).d;
    float[][] vals = rsf.d;
    if (complement)
      for   (int i1 = 0; i1 < n1; i1++)	       
	for (int i0 = 0; i0 < n0; i0++) {
	  mskd[i1][i0] = (vals[i1][i0] != 0F) ? 0F:1F;
	}
    else
      for   (int i1 = 0; i1 < n1; i1++)	       
	for (int i0 = 0; i0 < n0; i0++) {
	  mskd[i1][i0] = (vals[i1][i0] == 0F) ? 0F:1F;
	}
  }
  
  void createMask3(Rsf3 rsf, boolean complement) {
    int n0 = ((RsfSpace) rsf.getSpace()).getNSamples(0);
    int n1 = ((RsfSpace) rsf.getSpace()).getNSamples(1);
    int n2 = ((RsfSpace) rsf.getSpace()).getNSamples(2);
    float[][][] vals = rsf.d;
    float[][][] mskd = ((Rsf3) msk).d;
    if (complement)
      for     (int i2 = 0; i2 < n2; i2++)	       
	for   (int i1 = 0; i1 < n1; i1++)	       
	  for (int i0 = 0; i0 < n0; i0++) {
	    mskd[i2][i1][i0] = (vals[i2][i1][i0] != 0F) ? 0F:1F;
	  }
    else
      for     (int i2 = 0; i2 < n2; i2++)	       
	for   (int i1 = 0; i1 < n1; i1++)	       
	  for (int i0 = 0; i0 < n0; i0++) {
	    mskd[i2][i1][i0] = (vals[i2][i1][i0] == 0F) ? 0F:1F;
	  }
  }
  
  void createMask4(Rsf4 rsf, boolean complement) {
    int n0 = ((RsfSpace) rsf.getSpace()).getNSamples(0);
    int n1 = ((RsfSpace) rsf.getSpace()).getNSamples(1);
    int n2 = ((RsfSpace) rsf.getSpace()).getNSamples(2);
    int n3 = ((RsfSpace) rsf.getSpace()).getNSamples(3);
    float[][][][] vals = rsf.d;
    float[][][][] mskd = ((Rsf4) msk).d;
    if (complement)
      for	(int i3 = 0; i3 < n3; i3++)	       
	for     (int i2 = 0; i2 < n2; i2++)	       
	  for   (int i1 = 0; i1 < n1; i1++)	       
	    for (int i0 = 0; i0 < n0; i0++) {
	      mskd[i3][i2][i1][i0] =
		(vals[i3][i2][i1][i0] != 0F) ? 0F:1F;
	    }
    else
      for	(int i3 = 0; i3 < n3; i3++)	       
	for     (int i2 = 0; i2 < n2; i2++)	       
	  for   (int i1 = 0; i1 < n1; i1++)	       
	    for (int i0 = 0; i0 < n0; i0++) {
	      mskd[i3][i2][i1][i0] =
		(vals[i3][i2][i1][i0] == 0F) ? 0F:1F;
	    }
  }
  
  void createMask5(Rsf5 rsf, boolean complement) {
    int n0 = ((RsfSpace) rsf.getSpace()).getNSamples(0);
    int n1 = ((RsfSpace) rsf.getSpace()).getNSamples(1);
    int n2 = ((RsfSpace) rsf.getSpace()).getNSamples(2);
    int n3 = ((RsfSpace) rsf.getSpace()).getNSamples(3);
    int n4 = ((RsfSpace) rsf.getSpace()).getNSamples(4);
    float[][][][][] vals = rsf.d;
    float[][][][][] mskd = ((Rsf5) msk).d;
    if (complement)
      for   	  (int i4 = 0; i4 < n4; i4++)	       
	for	  (int i3 = 0; i3 < n3; i3++)	       
	  for     (int i2 = 0; i2 < n2; i2++)	       
	    for   (int i1 = 0; i1 < n1; i1++)	       
	      for (int i0 = 0; i0 < n0; i0++) {
		mskd[i4][i3][i2][i1][i0] =
		  (vals[i4][i3][i2][i1][i0] != 0F) ? 0F:1F;
	      }
    else
      for   	  (int i4 = 0; i4 < n4; i4++)	       
	for	  (int i3 = 0; i3 < n3; i3++)	       
	  for     (int i2 = 0; i2 < n2; i2++)	       
	    for   (int i1 = 0; i1 < n1; i1++)	       
	      for (int i0 = 0; i0 < n0; i0++) {
		mskd[i4][i3][i2][i1][i0] =
		  (vals[i4][i3][i2][i1][i0] == 0F) ? 0F:1F;
	      }
  }
  
  void createMask6(Rsf6 rsf, boolean complement) {
    int n0 = ((RsfSpace) rsf.getSpace()).getNSamples(0);
    int n1 = ((RsfSpace) rsf.getSpace()).getNSamples(1);
    int n2 = ((RsfSpace) rsf.getSpace()).getNSamples(2);
    int n3 = ((RsfSpace) rsf.getSpace()).getNSamples(3);
    int n4 = ((RsfSpace) rsf.getSpace()).getNSamples(4);
    int n5 = ((RsfSpace) rsf.getSpace()).getNSamples(5);
    float[][][][][][] vals = rsf.d;
    float[][][][][][] mskd = ((Rsf6) msk).d;
    if (complement)
      for           (int i5 = 0; i5 < n5; i5++)
	for   	    (int i4 = 0; i4 < n4; i4++)	       
	  for	    (int i3 = 0; i3 < n3; i3++)	       
	    for     (int i2 = 0; i2 < n2; i2++)	       
	      for   (int i1 = 0; i1 < n1; i1++)	       
		for (int i0 = 0; i0 < n0; i0++) {
		  mskd[i5][i4][i3][i2][i1][i0] =
		    (vals[i5][i4][i3][i2][i1][i0] != 0F) ? 0F:1F;
		}
    else
      for           (int i5 = 0; i5 < n5; i5++)
	for   	    (int i4 = 0; i4 < n4; i4++)	       
	  for	    (int i3 = 0; i3 < n3; i3++)	       
	    for     (int i2 = 0; i2 < n2; i2++)	       
	      for   (int i1 = 0; i1 < n1; i1++)	       
		for (int i0 = 0; i0 < n0; i0++) {
		  mskd[i5][i4][i3][i2][i1][i0] =
		    (vals[i5][i4][i3][i2][i1][i0] == 0F) ? 0F:1F;
		}
  }
  
  protected void apply(boolean add, Vector dom, Vector ran) {
    if (add) ((Rsf) ran).addMul((Rsf) dom, msk);
    else     ((Rsf) ran).mul((Rsf) dom, msk);
  }
  
  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (! (obj2 instanceof SepMask)) return false;
    SepMask msk2 = (SepMask) obj2;
    return (super.equals(msk2) &&
	    this.msk == msk2.msk);
  }
  
  public JamObject  copy() { return new SepMask(this); }
  
  public String toString() {
    return ("SepMask:\n\tMask:\n" +
	    msk);
  }
}
