package  rsf.util;

import  sep.io.*;
import  rsf.vector.*;
import java.util.*;

/** 
 * This computes a histogram of a given Rsf. 
 * @param n1=10          number of amplitude bins.
 * @param o1=[minOfData] smallest amplitude bin computed 
 * @param d1=[(maxOfData -  minOfData)/n1] bin increment
 * 
 * I would like to be able to set these values ...
 * fData.set("label2","Number_of_Samples");
 *  fData.set("title","Histogram");
 *
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class Histogram {
  public static void main(String[] args) {
    ArgumentKeeper command = new ArgumentKeeper(args);
    Rsf input = RsfFactory.newRsf(System.in);
    // find output params
    int n1 = command.getInt("n1","10");

    Attr attributes = new Attr(input); 
    float maxV      = attributes.getMax();
    float minV      = attributes.getMin();
    
    float inc = (maxV - minV + 1) / n1; 
    float d1  = command.getFloat("d1",inc); 
    float o1  = command.getFloat("o1",minV);

    String label1 = command.getString("label1","Amplitude");
    String unit1  = command.getString("unit1","Give it a unit");
    
    // Create the output vector, sort the amplitudes
    Axis[] axisArray = new Axis[1];
    axisArray[0]     = new Axis(n1,d1,o1,label1,unit1); 
    Rsf1  bin  = (Rsf1) RsfFactory.newRsf(axisArray);
    sort(input,bin); 
    bin.setTitle("Histogram");
    bin.write(System.out); 
  }
  /** 
   * input is sorted into bin. The bin Rsf is sampling the input's amplitude 
   * space. 
   */
  public static void sort(Rsf input, Rsf1 bin) {
    int    dim = input.getDimension();
    switch (dim) {
    case   1:
      sort1D((Rsf1) input, bin); 
      break;
    case   2:
      sort2D((Rsf2) input, bin); 
      break;
    case   3:
      sort3D((Rsf3) input, bin); 
      break;
    default:
      System.err.println("Error: only supported" +
			 " up to " + 3 + " dimensions");
    }
  }

  static void sort1D(Rsf1 ipt, Rsf1 bin) {
    Axis binAxis = ((RsfSpace) bin.getSpace()).getAxes()[0];
    float   bo = binAxis.offset;
    float   bd = binAxis.delta;
    float[] bb = bin.d;  int   bn = binAxis.nsamples; 
    float[] dd = ipt.d;  int[] dn = ipt.getNSamples(); 

    int index = 0; float incrm = 1/bd; 

    for(int i0=0; i0 < dn[0]; i0++) {
      index = (int)((dd[i0]-bo) * incrm);
      if (index > -1 && index < bn) {
	bb[index] += 1F;
      }
    }
  }

  static void sort2D(Rsf2 ipt, Rsf1 bin) {
    Axis binAxis = ((RsfSpace) bin.getSpace()).getAxes()[0];
    float     bo = binAxis.offset;
    float     bd = binAxis.delta;
    float[]   bb = bin.d;  int   bn = binAxis.nsamples; 
    float[][] dd = ipt.d;  int[] dn = ipt.getNSamples(); 

    int index = 0; float incrm = 1/bd; 

    for(int i1=0; i1 < dn[1]; i1++) {
    for(int i0=0; i0 < dn[0]; i0++) {
      index = (int)((dd[i1][i0]-bo) * incrm);
      if (index > -1 && index < bn) {
	bb[index] += 1F;
      }
    }}
  }

  static void sort3D(Rsf3 ipt, Rsf1 bin) {
    Axis binAxis = ((RsfSpace) bin.getSpace()).getAxes()[0];
    float       bo = binAxis.offset;
    float       bd = binAxis.delta;
    float[]     bb = bin.d;  int   bn = binAxis.nsamples; 
    float[][][] dd = ipt.d;  int[] dn = ipt.getNSamples(); 

    int index = 0; float incrm = 1/bd; 

    for(int i2=0; i2 < dn[2]; i2++) {
    for(int i1=0; i1 < dn[1]; i1++) {
    for(int i0=0; i0 < dn[0]; i0++) {
      index = (int)((dd[i2][i1][i0]-bo) * incrm);
      if (index > -1 && index < bn) {
	bb[index] += 1F;
      }
    }}}
    
  }
}
    
