package   rsf.util;

import 	  jam.vector.*;
import 	  jam.operator.*;
import 	  rsf.vector.*;
import    sep.io.*;
import    java.util.StringTokenizer;

/**
 * Generalizes the Laplacian gnerator.
 * 
 * Maybe this should have only a single central peak: easier implementation.
 * This should look like Box, Tent etc. Problem: how to specify second 
 * Rsf easily. Probalby bigRsf defined by Rsf (simulate newRsf() functions). 
 * smlRsf specified by array of samples. Simple check method that nsamples 
 * is odd and that smlRsf fits into bigRsf. 
 *
 * This should use Window-Patch: Window-Patch has to use to general 
 * rsf.operator? Generalisation of Window?! 
 *
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class LapGeneral { 
  /**
   * @param <pre>bigFilt=5,3,2 smlFilt=2,1,1<pre> to hand in 2 filters of 
   * dimension 3. The filter length is 2n-1 where n is the length specified 
   * in the command line. 
   */
  public static void main(String[] args) { 
    ArgumentKeeper command = new ArgumentKeeper(args);

    int[] nBg = command.getIntArray("bigFilt"); 
    int[] nSm = command.getIntArray("smlFilt"); 
    for(int i=0; i< nBg.length; i++) nBg[i] = 2 * nBg[i] - 1; 
    for(int i=0; i< nSm.length; i++) nSm[i] = 2 * nSm[i] - 1; 

    int dim = nBg.length;
    if (nBg.length != nSm.length) 
      System.err.println("Dimension do not match!");

    for (int i=0; i < dim; i++) {
      if(nBg[i] < nSm[i]) 
	System.err.println("NSample[" + i +"] do not match!");
    } 

    Rsf big  = RsfFactory.newRsf(nBg);
    Rsf sml  = RsfFactory.newRsf(nSm);

    getLapGeneral(big, sml); 

    if (command.containsKey("file")) {
      big.write(command.getString("file"));
    }
    else {
      big.write(System.out); 
    }
  }

  public static void getLapGeneral(Rsf big, Rsf sml) {

    Tent.setTent(big);
    Tent.setTent(sml);
    big.neg();

    // this would be much nicer with adjoint Window (zero-padding) operator
    // should I use them here? probably. 

    int o1 = 0; 
    int o2 = 0; 
    int o3 = 0;

    int[] nn = ((RsfSpace) sml.getSpace()).getNSamples();
    int[] mm = ((RsfSpace) big.getSpace()).getNSamples();
    switch (nn.length) {
    case 1: 
      o1 = (mm[0] - nn[0]) / 2;
      float[] x1 = ((Rsf1) big).d;
      float[] y1 = ((Rsf1) sml).d;
      for (int i1=0; i1<nn[0]; i1++) {
	x1[o1+i1] += y1[i1];
      }
      break;
    case 2: 
      o2 = (mm[1] - nn[1]) / 2;
      o1 = (mm[0] - nn[0]) / 2;
      float[][] x2 = ((Rsf2) big).d;
      float[][] y2 = ((Rsf2) sml).d;
      for (int i2=0; i2<nn[1]; i2++) {
      for (int i1=0; i1<nn[0]; i1++) {
	x2[o2+i2][o1+i1] += y2[i2][i1];
      }}
      break;
    case 3: 
      o3 = (mm[2] - nn[2]) / 2;
      o2 = (mm[1] - nn[1]) / 2;
      o1 = (mm[0] - nn[0]) / 2;
      float[][][] x3 = ((Rsf3) big).d;
      float[][][] y3 = ((Rsf3) sml).d;
      for (int i3=0; i3<nn[2]; i3++) {
      for (int i2=0; i2<nn[1]; i2++) {
      for (int i1=0; i1<nn[0]; i1++) {
	x3[o3+i3][o2+i2][o1+i1] += y3[i3][i2][i1];
      }}}
      break;
    }
  }
}







