package   rsf.util;

import 	  jam.vector.*;
import 	  jam.operator.*;
import 	  rsf.vector.*;
import    sep.io.*; 

/**
 * <BR><!--#include file="./source.inc"--><BR>
 * 
 * 
 */
final public class Laplacian {
  public static void main(String[] args) {
    ArgumentKeeper cmd = new ArgumentKeeper(args);
    Rsf filt = newRsf(cmd.getInt("dim",3));
    if (cmd.containsKey("file")) {
      filt.write(cmd.getString("file"));
    }
    else {
      filt.write(System.out); 
    }
  }
  static void testSpace(RsfSpace mySpace) {
    String errorOfLength = "Error: Rsf dimension and length incompatible";
    switch (mySpace.getDimension()) {
    case 1:
      if (mySpace.getAxis(0).nsamples != 3) 
	System.err.println(errorOfLength);
      break;
    case 2:
      if (mySpace.getAxis(0).nsamples != 3 || 
	  mySpace.getAxis(1).nsamples != 3 ) 
	System.err.println(errorOfLength);
      break;
    case 3:
      if (mySpace.getAxis(0).nsamples != 3 ||
	  mySpace.getAxis(1).nsamples != 3 ||
	  mySpace.getAxis(2).nsamples != 3 ) 
	System.err.println(errorOfLength);
      break;
    default:
      System.err.println("Error: Dimension not implemented yet");
    }
  }

  public static Rsf newRsf(Axis[] axes   ) {
    return newRsf(new RsfSpace(axes));
  }
  public static Rsf newRsf(int dim) {
    Axis[] axes = new Axis[dim];
    for (int i=0; i<dim; i++) 
      axes[i] = new Axis(3,1.f,0.f,"","");
    return newRsf(axes);
  }
  public static Rsf newRsf(RsfSpace filtSpace) {
    testSpace(filtSpace); 
    Rsf filt = RsfFactory.newRsf(filtSpace); 

    switch (filt.getDimension()) {
    case 1: 
      ((Rsf1) filt).d[1]       = -2f; 
      ((Rsf1) filt).d[0]       =  1f; 
      ((Rsf1) filt).d[2]       =  1f; 
      break; 
    case 2:
      ((Rsf2) filt).d[1][1]    = -4f; 
      ((Rsf2) filt).d[0][1]    =  1f; 
      ((Rsf2) filt).d[1][0]    =  1f; 
      ((Rsf2) filt).d[1][2]    =  1f; 
      ((Rsf2) filt).d[2][1]    =  1f; 
      break; 
    case 3:
      ((Rsf3) filt).d[1][1][1] = -6f; 
      ((Rsf3) filt).d[0][1][1] =  1f; 
      ((Rsf3) filt).d[2][1][1] =  1f; 
      ((Rsf3) filt).d[1][0][1] =  1f; 
      ((Rsf3) filt).d[1][1][0] =  1f; 
      ((Rsf3) filt).d[1][1][2] =  1f; 
      ((Rsf3) filt).d[1][2][1] =  1f; 
      break;
    default:
      filt = null; 
      System.err.println("Only implemented for 3 dimensions ...");
    }
    return filt;
  }
}

