package  rsf.util;

import   sep.io.*;
import   rsf.vector.*;
import  java.util.Random;
/**
 * This selects n samples from the input Rsf and returns a 1-D Rsf with 
 * those samples. Usually used when computing a histogram. 
 * 
 * @param samples=10 number of random samples. 
 * 
 * <BR><!--#include file="./source.inc"--><BR>
 */
public class RandomSelector {
  public static void main(String[] args) {
    ArgumentKeeper command = new ArgumentKeeper(args);
    Rsf input = RsfFactory.newRsf(System.in);
    int samples = command.getInt("samples","10"); 
    // Select random indices, set output data, write data to stdout 
    Random hiky = new Random();

    Rsf ouput = select(input, samples, hiky);
      ouput.write(System.out);
  }

  public static Rsf select(Rsf input, int samples, Random hiky) {
    Rsf1 ouput = (Rsf1) RsfFactory.newRsf(samples); 
    int    dim = input.getDimension();
    switch (dim) {
    case   1:
      select1D((Rsf1) input, ouput, hiky); 
      break;
    case   2:
      select2D((Rsf2) input, ouput, hiky); 
      break;
    case   3:
      select3D((Rsf3) input, ouput, hiky); 
      break;
    default:
      System.err.println("Error: only supported" +
			 " up to " + 3 + " dimensions");
    }
    return ouput; 
  }

  static void select1D(Rsf1 input, Rsf1 ouput, Random hiky) {
    int[] oo = ouput.getNSamples(); float[] od = ouput.d;  
    int[] ii = input.getNSamples(); float[] id = input.d;  
    for (int i=0; i < oo[0]; i++) {
      ouput.d[i] = input.d[(int)(hiky.nextFloat()*ii[0])];
    }
  }
  static void select2D(Rsf2 input, Rsf1 ouput, Random hiky) {
    int[] oo = ouput.getNSamples(); float[]   od = ouput.d;  
    int[] ii = input.getNSamples(); float[][] id = input.d;  
    for (int i=0; i < oo[0]; i++) {
      ouput.d[i] = input.d[(int)(hiky.nextFloat()*ii[1])]
	                  [(int)(hiky.nextFloat()*ii[0])];
    }
  }
  static void select3D(Rsf3 input, Rsf1 ouput, Random hiky) {
    int[] oo = ouput.getNSamples(); float[]     od = ouput.d;  
    int[] ii = input.getNSamples(); float[][][] id = input.d;  
    for (int i=0; i < oo[0]; i++) {
      ouput.d[i] = input.d[(int)(hiky.nextFloat()*ii[2])]
	                  [(int)(hiky.nextFloat()*ii[1])]
	                  [(int)(hiky.nextFloat()*ii[0])];
    }
  }
}
 
    
      
