package   rsf.util;

import 	  jam.vector.*;
import 	  jam.operator.*;
import 	  rsf.vector.*;
import    sep.io.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * Computes an Rsf with a Ricker wavelet. 
 * The Ricker volume is normalized to 1.
 * The Ricker formula is (1 - 2 x^2) Exp(-x^2) where x is the distance from 
 * the center. This is stretching the concept of a static class and might 
 * be better of as an object.  
 */
final public class Ricker {
  /**
   * @param <pre>nsamples=1,2,3<pre> etc specifies the size of the filter.
   */
  public static void main(String[] args) {
    ArgumentKeeper cmd = new ArgumentKeeper(args);
    int[] nsamples = cmd.getIntArray("nsamples"); 
    Rsf ricky = Ricker.newRsf(RsfFactory.newRsf(nsamples)); 
    if (cmd.containsKey("file")) {
      ricky.write(cmd.getString("file"));
    }
    else {
      ricky.write(System.out); 
    }
  }
  public static Rsf newRsf(   Rsf src) {
    Rsf ricky  = RsfFactory.newRsf(src);
    setRicker(ricky);
    return ricky;
  } 

  public static void setRicker(Rsf ricky) {
    int   dim = ricky.getDimension();
    switch (dim) {
    case   1:
      setRicker((Rsf1) ricky);
      break;
    case   2:
      setRicker((Rsf2) ricky);
      break;
    case   3:
      setRicker((Rsf3) ricky);
      break;
    default:
      System.err.println("Error: Window only supported" +
			 " up to " + 3 + " dimensions");
    }
  }

  static float getAmplitude(double relativeDistance) {
    double argu1, argu2, argu3, ampli, ratio, omega, phase; 
    omega  = 2. * Math.PI * Ricker.getCenterFrequency();
    phase  = omega * relativeDistance; 
    argu1  = phase               ; argu1 = argu1*argu1;
    argu2  = phase + Math.PI / 2.; argu2 = argu2*argu2;
    argu3  = phase - Math.PI / 2.; argu3 = argu3*argu3;
    ampli  =       (1. - 2. * argu1) * Math.exp(-argu1);
    //ampli += -.5 * (1. - 2. * argu2) * Math.exp(-argu2);
    //ampli += -.5 * (1. - 2. * argu3) * Math.exp(-argu3);
    return (float) ampli;
  }
  /** 
   * @param   length length of data in samples.
   * @return  center frequency in dt=1. 
   */
  static double getCenterFrequency(          ) {
    return Ricker.getCenterFrequency(1./1000.); 
  }
  static double getCenterFrequency(double ratio) {
    return (Math.sqrt(- Math.log(ratio)) / Math.PI); 
  }

  static void setRicker(Rsf1 ricky) {
    float sum = 0.f; 
    int[] nn  = ricky.getNSamples();
    for(int i0=0; i0 < nn[0]; i0++){ double d0 = ((i0+.5)/nn[0]) - 0.5;
      ricky.d[i0] = Ricker.getAmplitude(d0);
      sum         += ricky.d[i0] * ricky.d[i0];
    }
    sum = (float) Math.sqrt(sum); 
    for(int i0=0; i0 < nn[0]; i0++) ricky.d[i0]         /= sum;
  }
  static void setRicker(Rsf2 ricky) {
    float sum = 0.f; 
    int[] nn  = ricky.getNSamples();
    for(int i1=0; i1 < nn[1]; i1++){ double d1 = ((i1+.5)/nn[1]) - 0.5;
    for(int i0=0; i0 < nn[0]; i0++){ double d0 = ((i0+.5)/nn[0]) - 0.5;
      ricky.d[i1][i0]   =  Ricker.getAmplitude(Math.sqrt(d1*d1+d0*d0)); 
      sum              += ricky.d[i1][i0] * ricky.d[i1][i0];
    }}
    sum = (float) Math.sqrt(sum); 
    for(int i1=0; i1 < nn[1]; i1++) 
    for(int i0=0; i0 < nn[0]; i0++) ricky.d[i1][i0]     /= sum;
  }
  static void setRicker(Rsf3 ricky) {
    float sum = 0.f; 
    int[] nn  = ricky.getNSamples();
    for(int i2=0; i2 < nn[2]; i2++){ double d2 = ((i2+.5)/nn[2]) - 0.5;
    for(int i1=0; i1 < nn[1]; i1++){ double d1 = ((i1+.5)/nn[1]) - 0.5;
    for(int i0=0; i0 < nn[0]; i0++){ double d0 = ((i0+.5)/nn[0]) - 0.5;
      ricky.d[i2][i1][i0] = Ricker.getAmplitude(Math.sqrt(d2*d2+d1*d1+d0*d0)); 
      sum                 += ricky.d[i2][i1][i0] * ricky.d[i2][i1][i0];
    }}}
    sum = (float) Math.sqrt(sum); 
    for(int i2=0; i2 < nn[2]; i2++) 
    for(int i1=0; i1 < nn[1]; i1++) 
    for(int i0=0; i0 < nn[0]; i0++) ricky.d[i2][i1][i0] /= sum;
  }
}

