package   rsf.util;

import 	  jam.vector.*;
import 	  jam.operator.*;
import 	  rsf.vector.*;
import    sep.io.*;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * Computes an Rsf with a tent weight. The tent volume is normalized to 1. 
 * (This does not use the convolution of 2 box filters, but it could)
 */
final public class Tent {
  /**
   * @param <pre>nsamples=1,2,3<pre> etc specifies the size of the filter.
   */
  public static void main(String[] args) {
    ArgumentKeeper cmd = new ArgumentKeeper(args);
    int[] nsamples = cmd.getIntArray("nsamples"); 
    Rsf tent = Tent.newRsf(nsamples); 
    if (cmd.containsKey("file")) {
      tent.write(cmd.getString("file"));
    }
    else {
      tent.write(System.out); 
    }
  }

  public static Rsf newRsf(Axis[] axes) {
    Rsf tent  = RsfFactory.newRsf(axes);
    setTent(tent);
    return tent;
  }
  public static Rsf newRsf(      int n0) {
    Rsf tent  = RsfFactory.newRsf(   n0);
    setTent(tent);
    return tent;
  }
  public static Rsf newRsf(      int n1, int n0) {
    Rsf tent  = RsfFactory.newRsf(   n1,     n0);
    setTent(tent);
    return tent;
  }
  public static Rsf newRsf(      int n2, int n1, int n0) {
    Rsf tent  = RsfFactory.newRsf(   n2,     n1,     n0);
    setTent(tent);
    return tent;
  }
  public static Rsf newRsf(int[]  nsamples) {
    Rsf tent  = RsfFactory.newRsf(nsamples);
    setTent(tent);
    return tent;
  }
  public static Rsf newRsf(   Rsf src) {
    Rsf tent  = RsfFactory.newRsf(src);
    setTent(tent);
    return tent;
  } 

  public static void setTent(Rsf tent) {
    int   dim = tent.getDimension();
    switch (dim) {
    case   1:
      setTent((Rsf1) tent);
      break;
    case   2:
      setTent((Rsf2) tent);
      break;
    case   3:
      setTent((Rsf3) tent);
      break;
    default:
      System.err.println("Error: Window only supported" +
			 " up to " + 3 + " dimensions");
    }
  }

  // nice formulation for cheop in Jon's book tdf
  // the normalisation should be done more efficiently
  static void setTent(Rsf1 tent) {
    float sum = 0.f; 
    int[] nn  = tent.getNSamples();
    for(int i0=0; i0 < nn[0]; i0++) { int v0 = 1+nn[0]-Math.abs(2*i0-nn[0]+1); 
      tent.d[i0]         =                   v0  * 0.5f;
      sum               +=                   v0  * 0.5f;
    }
    for(int i0=0; i0 < nn[0]; i0++) tent.d[i0]         /= sum; 
  }
  static void setTent(Rsf2 tent) {
    float sum = 0.f; 
    int[] nn  = tent.getNSamples();
    for(int i1=0; i1 < nn[1]; i1++) { int v1 = 1+nn[1]-Math.abs(2*i1-nn[1]+1); 
    for(int i0=0; i0 < nn[0]; i0++) { int v0 = 1+nn[0]-Math.abs(2*i0-nn[0]+1); 
      tent.d[i1][i0]     =  (((long)     v1)*v0) * 0.25f; 
      sum               +=  (((long)     v1)*v0) * 0.25f; 
    }}
    for(int i1=0; i1 < nn[1]; i1++) 
    for(int i0=0; i0 < nn[0]; i0++) tent.d[i1][i0]     /= sum;
  }
  static void setTent(Rsf3 tent) {
    float sum = 0.f; 
    int[] nn  = tent.getNSamples();
    for(int i2=0; i2 < nn[2]; i2++) { int v2 = 1+nn[2]-Math.abs(2*i2-nn[2]+1); 
    for(int i1=0; i1 < nn[1]; i1++) { int v1 = 1+nn[1]-Math.abs(2*i1-nn[1]+1); 
    for(int i0=0; i0 < nn[0]; i0++) { int v0 = 1+nn[0]-Math.abs(2*i0-nn[0]+1); 
      tent.d[i2][i1][i0] = ((((long) v2)*v1)*v0) * 0.125f; 
      sum               += ((((long) v2)*v1)*v0) * 0.125f; 
    }}}
    for(int i2=0; i2 < nn[2]; i2++) 
    for(int i1=0; i1 < nn[1]; i1++) 
    for(int i0=0; i0 < nn[0]; i0++) tent.d[i2][i1][i0] /= sum;
  }
}

