package  rsf.vector;

import   sep.io.*;
import   jam.vector.*;
import juice.util.JuiceUtil;
import  java.io.*;
import  java.net.*;
import  java.util.Random;
import   jam.JamObject;

/**
 *  Regularly Sampled Function
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * Often I want to treat all Rsf without knowing their dimensions.
 * I think I want an Enumeration object of the Rsf.<BR>
 * I need to coherently define the axis naming: getAxis(0) refers to fastest
 * Axis. data[ax[1].nsamples, ax[0].nsamples]. Is that intuitive? natural for
 * Java?  
 *
 * @see       RsfSpace
 * @author    Joel Schroeder, Matt Schwab
 */

public abstract class Rsf implements Vector {
  protected RsfSpace space;
  protected String   title = "Unknown";
  
  // constructors
  public Rsf() {}
  public Rsf(RsfSpace space) {
    this.space = space;
    this.allocateData();
  }
  Rsf(rsfF.vector.Rsf src) {
    this(new RsfSpace((rsfF.vector.RsfSpace) src.getSpace()));
    this.title = src.getTitle();
  }

  /** copy constructor */
  protected Rsf(Rsf src)     {
    this(src.space);
    this.title = src.title; 
    this.copyFrom(src);
  }

  abstract void allocateData();
  public abstract void copyFrom(Vector src);

  public boolean equals(Object obj2) {
    if (this == obj2) return true;
    if (!(obj2 instanceof Rsf)) return false;
    return (this.space.equals(((Rsf) obj2).space) &&
	    this.equalData(obj2));
  }

  /** we may assume that obj2 is an Rsf of the appropriate dimensions */
  abstract boolean equalData(Object obj2);

  /**
   * @param int[] i equals to internal java array storage: i[0] fastest.  
   * get/set Datum is for encapsulated access 
   * (alternative to direct access to x.d)
   */
  public abstract float getDatum(int[] i);
  public abstract void	setDatum(int[] i, float v);
  
  public int   getDimension() { return space.getDimension(); }

  /** depreciated: use getAxis(int i).nsamples; */
  public int[] getNSamples()  { return space.getNSamples(); }
  public int   size()         { return space.size();        }
  /** getAxis(0) is the fastest axis */ 
  public Axis getAxis(int i)  { return space.getAxis(i);    }

  // Fill
  public void zero() { fill(0); }
  public abstract void fill(float a);
  public abstract void rand();
  public abstract void rand(Random r);

  /** this = this x vec2 (point-wise multiplication) */
  public abstract void mul(Vector vec2);
  /** this = vec1 x vec2 (point-wise multiplication) */
  public abstract void mul(Vector vec1, Vector vec2);
  /** this = this + vec1 x vec2 (point-wise multiplication) */
  public abstract void addMul(Vector vec1, Vector vec2);
  /** this = vec1 + vec2 x vec3 (point-wise multiplication) */
  public abstract void addMul(Vector vec1, Vector vec2, Vector vec3);
  
  public float norm    () { return (float) Math.sqrt(norm2()); }
  public Space getSpace() { return space; }

  // I/O
  public String toString() { 
    return ("Rsf: " + getTitle() + "\nSpace:\n" + getSpace()); 
  }

  /** 
   * The name of a data set changes as it is processed: consequently, 
   * a programmer should be able to conveniently set it. The name is 
   * property of the data itself, not of the space and or axes. 
   * However, the title is not part of the Rsf method r1.equals(Rsf r2)
   */ 
  public String getTitle(            ) { return this.title ; }
  public void   setTitle(String title) { this.title = title; }

  public void write  (String fileName) {
    try {
      this.write(new FileOutputStream(new File(fileName)));
    } catch(Exception e) { System.err.println(e); }
  }

  protected ParaKeeper getPara() {
    //    RsfFactory.rsf2rsfF(this).write(outstr);
    int dim = space.getDimension();
    ParaKeeper pars = new ParaKeeper();
    pars.set("in" , "\"stdin\"");
    pars.set("dim", dim);
    pars.set("title", this.title); 
    pars.set("label", space.getDataAxis().label); 
    pars.set("unit" , space.getDataAxis().unit );
    for (int i = 1; i <= dim; i++ ) {
      pars.set("o"     + i, space.getAxes()[i-1].offset   );
      pars.set("d"     + i, space.getAxes()[i-1].delta    );
      pars.set("n"     + i, space.getAxes()[i-1].nsamples );
      pars.set("label" + i, space.getAxes()[i-1].label    );
      pars.set("unit"  + i, space.getAxes()[i-1].unit     );
    }
    return pars; 
  }

  public abstract void write (OutputStream outstr); 

}

