package  rsf.vector;

import   sep.io.*;
import   jam.vector.*;
import juice.util.JuiceUtil;
import  java.io.*;
import  java.net.*;
import  java.util.Random;
import   jam.JamObject;

/*
  Don't forget to check for space compatibility -- maybe this could
  be handled in a superclass.
  */

/**
 *  Regularly Sampled Function (in 1 dimension)
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @see       RsfSpace
 * @author    Joel Schroeder, Matt Schwab
 */

public class Rsf1 extends Rsf {
  int                size;
  public    float[]  d; 
  
  // constructors
  public Rsf1() {}

  public Rsf1(RsfSpace space) { super(space); }
  Rsf1(rsfF.vector.Rsf src) {
    super(src);
    this.d = src.d;
  }

  /** copy constructor */
  protected Rsf1(Rsf1 src)    { super(src); }
  
  protected void allocateData() {
    size = space.size();
    d = new float[space.size()];
  }
  
  /**
   * WARNING: the space copied is a shallow copy, meaning
   * that if you subsequently change any of the internals of the
   * space of one vector, then the space of the other vector is
   * affected.
   */
  public JamObject copy() { return new Rsf1(this); }

  public void copyFrom(Vector src) {
    for (int i = 0; i < size; i++ )
      this.d[i] = ((Rsf1) src).d[i];
  }

  public boolean equalData(Object obj2) {
    return JuiceUtil.equalArrays(this.d, ((Rsf1) obj2).d);
  }
  
  public float[] getData() { return d; }
  /*
  public void    setData(float[] data) {
    if (data.length != size) {
      System.err.println("Error in Rsf.setData(): size of new data is " +
			 "incompatible with the RsfSpace.");
      return;
    }
    d = data;
  }
  */

  /**
   * get/set Datum is for encapsulated access (alternative to direct
   * access to x.d)
   */
  public float getDatum(int[] i) { return d[i[0]]; }
  public float getDatum(int   i) { return d[i]   ; }
  
  public void  setDatum(int[] i, float v) { d[i[0]] = v; }
  public void  setDatum(int   i, float v) { d[i]    = v; }

  public void fill(float a) {
    for (int i = 0; i < size; i++)
      d[i] = a;
  }
  public void rand() {
    for (int i = 0; i < size; i++ )
      this.d[i] = 2*((float) Math.random()) - 1.f;
  }
  public void rand(Random r) {
    for (int i = 0; i < size; i++ )
      this.d[i] = 2*(r.nextFloat()) - 1.f;
  }

  public void mul(Vector vec2) {
    for (int i = 0; i < size; i++)
      this.d[i] *= ((Rsf1) vec2).d[i];
  }
  public void mul(Vector vec1, Vector vec2) {
    for (int i = 0; i < size; i++)
      this.d[i] = ((Rsf1) vec1).d[i] * ((Rsf1) vec2).d[i];
  }
  public void addMul(Vector vec1, Vector vec2) {
    for (int i = 0; i < size; i++)
      this.d[i] += ((Rsf1) vec1).d[i] * ((Rsf1) vec2).d[i];
  }
  public void addMul(Vector vec1, Vector vec2, Vector vec3) {
    for (int i = 0; i < size; i++)
      this.d[i]  = ((Rsf1) vec1).d[i] +
	((Rsf1) vec2).d[i] * ((Rsf1) vec3).d[i];
  }

  // methods from Vector
  public void neg() {
    for (int i = 0; i < size; i++ )
      d[i] = - d[i];
  }
  
  public void neg(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for (int i = 0; i < size; i++ )
      this.d[i] = - ((Rsf1) vec2).d[i];
  }
  
  public void addScale(float a,
		       Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for (int i = 0; i < size; i++ )
      this.d[i] +=  a * ((Rsf1) vec2).d[i];
  }
  
  public void addScale(float a,
		       Vector vec1,
		       Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for (int i = 0; i < size; i++ )
      this.d[i] = ((Rsf1) vec1).d[i] + a * ((Rsf1) vec2).d[i];
  }
  
  public void add(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for (int i = 0; i < size; i++ )
      this.d[i] +=  ((Rsf1) vec2).d[i];
  }
  
  public void add(Vector vec1,
		  Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for (int i = 0; i < size; i++ )
      this.d[i] = ((Rsf1) vec1).d[i] + ((Rsf1) vec2).d[i];
  }
  public void sub(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for (int i = 0; i < size; i++ )
      this.d[i] -=  ((Rsf1) vec2).d[i];
  }
  
  public void sub(Vector vec1,
		  Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for (int i = 0; i < size; i++ )
      this.d[i] = ((Rsf1) vec1).d[i] - ((Rsf1) vec2).d[i];
  }
  
  public void scale(float        a    ) {
    for (int i = 0; i < size; i++ )
      this.d[i] *= a;
  }
  
  public void scale(float        a    ,
		    Vector vec ) {
    // throw if this and vec spaces are unequal
    for (int i = 0; i < size; i++ )
      this.d[i]  = a * ((Rsf1) vec).d[i];
  }
  
  // Inner product and related operations
  public float dot  (Vector vec2) {
    float innr = 0f;
    for (int i = 0; i < size; i++ )
      innr += this.d[i] * ((Rsf1) vec2).d[i];
    return innr;
  }
  
  public float norm2() {
    float norm2 = 0;
    for (int i = 0; i < size; i++ )
      norm2 += this.d[i] * this.d[i];
    return norm2;
  }
  
  // I/O
  public String toString() {
    return (super.toString() + "\n" +
	    "Sample data:"   + "\n" +
	    JuiceUtil.arrayToString(d, " "));
  }

  public void write (java.io.OutputStream outstr) {
    try {
      new RegularDataJ(d, getPara()).write(outstr);
    } catch(Exception e ) { System.err.println("Exception:" + e ); }
  }
}
