package  rsf.vector;

import   sep.io.*;
import   jam.vector.*;
import juice.util.JuiceUtil;
import  java.io.*;
import  java.net.*;
import  java.util.Random;
import   jam.JamObject;

/*
  Don't forget to check for space compatibility -- maybe this could
  be handled in a superclass.
  */

/**
 *  Regularly Sampled Function (in 2 dimensions)
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @see       RsfSpace
 * @author    Joel Schroeder, Matt Schwab
 */

public class Rsf2 extends Rsf {
  int              size0, size1;
  public float[][] d; 
  
  // constructors
  public Rsf2() {}

  public Rsf2(RsfSpace space) { super(space); }

  /** this looks defunct to me */
  Rsf2(rsfF.vector.Rsf src) {
    super(src);
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] = src.d[src.i(i,j)];
  }

  /** copy constructor */
  protected Rsf2(Rsf2 src)    { super(src); }
  
  protected void allocateData() {
    size0 = space.getNSamples(0);
    size1 = space.getNSamples(1);
    d = new float[size1][size0];
  }
  
  /**
   * WARNING: the space copied is a shallow copy, meaning
   * that if you subsequently change any of the internals of the
   * space of one vector, then the space of the other vector is
   * affected.
   */
  public JamObject copy() { return new Rsf2(this); }

  public void copyFrom(Vector src) {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] = ((Rsf2) src).d[j][i];
  }

  public boolean equalData(Object obj2) {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	if (this.d[j][i] != ((Rsf2) obj2).d[j][i]) return false;
    return true;
  }
  
  public float[][] getData() { return d; }

  /**
   * get/set Datum is for encapsulated access (alternative to direct
   * access to x.d)
   */
  public float getDatum(int[] i) { return d[i[1]][i[0]]; }
  public float getDatum(int   j, int i) { return d[j][i]; }
  
  public void  setDatum(int[] i, float v) { d[i[1]][i[0]]  = v; }
  public void  setDatum(int   j, int i, float v) { d[j][i] = v; }

  public void fill(float a) {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	d[j][i] = a;
  }
  public void rand() {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] = 2*((float) Math.random()) - 1.f;
  }
  public void rand(Random r) {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
      this.d[j][i] = 2*(r.nextFloat()) - 1.f;
  }

  public void mul(Vector vec2) {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] *=
	  ((Rsf2) vec2).d[j][i];
  }
  public void mul(Vector vec1, Vector vec2) {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] =
	  ((Rsf2) vec1).d[j][i] *
	  ((Rsf2) vec2).d[j][i];
  }
  public void addMul(Vector vec1, Vector vec2) {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] +=
	  ((Rsf2) vec1).d[j][i] *
	  ((Rsf2) vec2).d[j][i];
  }
  public void addMul(Vector vec1, Vector vec2, Vector vec3) {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i]  =
	  ((Rsf2) vec1).d[j][i] +
	  ((Rsf2) vec2).d[j][i] *
	  ((Rsf2) vec3).d[j][i];
  }
  
  // methods from Vector
  public void neg() {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	d[j][i] = - d[j][i];
  }
  
  public void neg(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] = - ((Rsf2) vec2).d[j][i];
  }
  
  public void addScale(float a,
		       Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] +=  a * ((Rsf2) vec2).d[j][i];
  }
  
  public void addScale(float a,
		       Vector vec1,
		       Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] = ((Rsf2) vec1).d[j][i] + a * ((Rsf2) vec2).d[j][i];
  }
  
  public void add(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] +=  ((Rsf2) vec2).d[j][i];
  }
  
  public void add(Vector vec1,
		  Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] = ((Rsf2) vec1).d[j][i] + ((Rsf2) vec2).d[j][i];
  }
  public void sub(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] -=  ((Rsf2) vec2).d[j][i];
  }
  
  public void sub(Vector vec1,
		  Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] = ((Rsf2) vec1).d[j][i] - ((Rsf2) vec2).d[j][i];
  }
  
  public void scale(float        a    ) {
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i] *= a;
  }
  
  public void scale(float        a    ,
		    Vector vec ) {
    // throw if this and vec spaces are unequal
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	this.d[j][i]  = a * ((Rsf2) vec).d[j][i];
  }
  
  // Inner product and related operations
  public float dot  (Vector vec2) {
    float innr = 0f;
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	innr += this.d[j][i] * ((Rsf2) vec2).d[j][i];
    return innr;
  }
  
  public float norm2() {
    float norm2 = 0;
    for   (int j = 0; j < size1; j++)
      for (int i = 0; i < size0; i++)
	norm2 += this.d[j][i] * this.d[j][i];
    return norm2;
  }
  
  // I/O
  public String toString() {
    String separator = " ";
    int n1 = space.getNSamples(1); if (n1 > 10) n1 = 10; 
    int n0 = space.getNSamples(0); if (n0 > 10) n0 = 10; 
    StringBuffer buf = new StringBuffer();
    buf.append(super.toString() + "\n");
    buf.append("Sample data: [i1]"   + "\n");
      for  (int i1=0; i1<n1; i1++) { buf.append("\n["+i1+"]: ");
	for(int i0=0; i0<n0; i0++) {
	  buf.append(d[i1][i0] + separator); 
    }}
    return buf.toString();
  }
  public void write (java.io.OutputStream outstr) {
    try {
      new RegularDataJ(d, getPara()).write(outstr);
    } catch(Exception e ) { System.err.println("Exception:" + e ); }
  }
}
