package  rsf.vector;

import   sep.io.*;
import   jam.vector.*;
import juice.util.JuiceUtil;
import  java.io.*;
import  java.net.*;
import  java.util.Random;
import   jam.JamObject;

/*
  Don't forget to check for space compatibility -- maybe this could
  be handled in a superclass.
  */

/**
 *  Regularly Sampled Function (in 5 dimensions)
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @see       RsfSpace
 * @author    Joel Schroeder, Matt Schwab
 */

public class Rsf5 extends Rsf {
  int                    size0, size1, size2, size3, size4;
  public float[][][][][] d; 
  
  // constructors
  public Rsf5() {}
  
  public Rsf5(RsfSpace space) { super(space); }
  Rsf5(rsfF.vector.Rsf src) {
    super(src);
    int[] i = new int[5];
    for         (i[4] = 0; i[4] < size4; i[4]++)
      for       (i[3] = 0; i[3] < size3; i[3]++)
	for     (i[2] = 0; i[2] < size2; i[2]++)
	  for   (i[1] = 0; i[1] < size1; i[1]++)
	    for (i[0] = 0; i[0] < size0; i[0]++)
	      this.d[i[4]][i[3]][i[2]][i[1]][i[0]] = src.d[src.i(i)];
  }
  
  /** copy constructor */
  protected Rsf5(Rsf5 src)    { super(src); }
  
  protected void allocateData() {
    size0 = space.getNSamples(0);
    size1 = space.getNSamples(1);
    size2 = space.getNSamples(2);
    size3 = space.getNSamples(3);
    size4 = space.getNSamples(4);
    d = new float[size4][size3][size2][size2][size1];
  }
  
  /**
   * WARNING: the space copied is a shallow copy, meaning
   * that if you subsequently change any of the internals of the
   * space of one vector, then the space of the other vector is
   * affected.
   */
  public JamObject copy() { return new Rsf5(this); }
  
  public void copyFrom(Vector src) {
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] = ((Rsf5) src).d[m][l][k][j][i];
  }
  
  public boolean equalData(Object obj2) {
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      if (this.d[m][l][k][j][i] != ((Rsf5) obj2).d[m][l][k][j][i]) return false;
    return true;
  }
  
  public float[][][][][] getData() { return d; }
  
  /**
   * get/set Datum is for encapsulated access (alternative to direct
   * access to x.d)
   */
  public float getDatum(int[] i) {
    return d[i[4]][i[3]][i[2]][i[1]][i[0]];
  }
  public float getDatum(int   m, int l, int k, int j, int i) {
    return   d[m][l][k][j][i];
  }
  
  public void  setDatum(int[] i, float v) {
    d[i[4]][i[3]][i[2]][i[1]][i[0]] = v;
  }
  public void  setDatum(int   m, int l, int k, int j, int i, float v) {
    d[m][l][k][j][i] = v;
  }
  
  public void fill(float a) {
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      d[m][l][k][j][i] = a;
  }
  public void rand() {
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] = 2*((float) Math.random()) - 1.f;
  }
  public void rand(Random r) {
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] = 2*(r.nextFloat()) - 1.f;
  }

  public void mul(Vector vec2) {
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] *=
		((Rsf5) vec2).d[m][l][k][j][i];
  }
  public void mul(Vector vec1, Vector vec2) {
      for         (int m = 0; m < size4; m++)
	for       (int l = 0; l < size3; l++)
	  for     (int k = 0; k < size2; k++)
	    for   (int j = 0; j < size1; j++)
	      for (int i = 0; i < size0; i++)
		this.d[m][l][k][j][i] =
		  ((Rsf5) vec1).d[m][l][k][j][i] *
		  ((Rsf5) vec2).d[m][l][k][j][i];
  }
  public void addMul(Vector vec1, Vector vec2) {
      for         (int m = 0; m < size4; m++)
	for       (int l = 0; l < size3; l++)
	  for     (int k = 0; k < size2; k++)
	    for   (int j = 0; j < size1; j++)
	      for (int i = 0; i < size0; i++)
		this.d[m][l][k][j][i] +=
		  ((Rsf5) vec1).d[m][l][k][j][i] *
		  ((Rsf5) vec2).d[m][l][k][j][i];
  }
  public void addMul(Vector vec1, Vector vec2, Vector vec3) {
      for         (int m = 0; m < size4; m++)
	for       (int l = 0; l < size3; l++)
	  for     (int k = 0; k < size2; k++)
	    for   (int j = 0; j < size1; j++)
	      for (int i = 0; i < size0; i++)
		this.d[m][l][k][j][i]  =
		  ((Rsf5) vec1).d[m][l][k][j][i] +
		  ((Rsf5) vec2).d[m][l][k][j][i] *
		  ((Rsf5) vec3).d[m][l][k][j][i];
  }
  
  // methods from Vector
  public void neg() {
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      d[m][l][k][j][i] = - d[m][l][k][j][i];
  }
  
  public void neg(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] = - ((Rsf5) vec2).d[m][l][k][j][i];
  }
  
  public void addScale(float a,
		       Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] += a * ((Rsf5) vec2).d[m][l][k][j][i];
  }
  
  public void addScale(float a,
		       Vector vec1,
		       Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] =
		((Rsf5) vec1).d[m][l][k][j][i] +
		a * ((Rsf5) vec2).d[m][l][k][j][i];
  }
  
  public void add(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] +=  ((Rsf5) vec2).d[m][l][k][j][i];
  }
  
  public void add(Vector vec1,
		  Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] =
		((Rsf5) vec1).d[m][l][k][j][i] +
		((Rsf5) vec2).d[m][l][k][j][i];
  }
  public void sub(Vector vec2) {
    // throw if this and vec2 spaces are unequal
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] -=  ((Rsf5) vec2).d[m][l][k][j][i];
  }
  
  public void sub(Vector vec1,
		  Vector vec2) {
    // throw if this and vec1, vec2 spaces are unequal
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] =
		((Rsf5) vec1).d[m][l][k][j][i] -
		((Rsf5) vec2).d[m][l][k][j][i];
  }
  
  public void scale(float        a    ) {
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i] *= a;
  }
  
  public void scale(float        a    ,
		    Vector vec) {
    // throw if this and vec spaces are unequal
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      this.d[m][l][k][j][i]  = a * ((Rsf5) vec).d[m][l][k][j][i];
  }
  
  // Inner product and related operations
  public float dot  (Vector vec2) {
    float innr = 0f;
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      innr += this.d[m][l][k][j][i] * ((Rsf5) vec2).d[m][l][k][j][i];
    return innr;
  }
  
  public float norm2() {
    float norm2 = 0;
    for         (int m = 0; m < size4; m++)
      for       (int l = 0; l < size3; l++)
	for     (int k = 0; k < size2; k++)
	  for   (int j = 0; j < size1; j++)
	    for (int i = 0; i < size0; i++)
	      norm2 += this.d[m][l][k][j][i] * this.d[m][l][k][j][i];
    return norm2;
  }
  
  // I/O
  public String toString() {
    String separator = " ";
    int n4 = space.getNSamples(4); if (n4 > 5) n4 = 5; 
    int n3 = space.getNSamples(3); if (n3 > 5) n3 = 5; 
    int n2 = space.getNSamples(2); if (n2 > 5) n2 = 5; 
    int n1 = space.getNSamples(1); if (n1 > 5) n1 = 5; 
    int n0 = space.getNSamples(0); if (n0 > 5) n0 = 5; 
    StringBuffer buf = new StringBuffer();
    buf.append(super.toString()             + "\n");
    buf.append("Sample data: [i4,i3,i2,i1]" + "\n");
    for        (int i4=0; i4<n4; i4++) { 
      for      (int i3=0; i3<n3; i3++) { 
	for    (int i2=0; i2<n2; i2++) { 
	  for  (int i1=0; i1<n1; i1++) { 
	    buf.append("\n["+i4+","+i3+","+i2+","+i1+"]: ");
	    for(int i0=0; i0<n0; i0++) {
	    buf.append(d[i4][i3][i2][i1][i0] + separator); 
    }}}}}
    return buf.toString();
  }
  public void write (java.io.OutputStream outstr) {
    try {
      new RegularDataJ(d, getPara()).write(outstr);
    } catch(Exception e ) { System.err.println("Exception:" + e ); }
  }
}
