package  rsf.vector;

import   jam.vector.*;
import juice.util.JuiceUtil;
import  java.io.*;
import  java.net.*;
import  java.util.Random;
import   jam.JamObject;

/**
 * <BR><!--#include file="./source.inc"--><BR>
 *
 * @see       Rsf
 * @see       RsfSpace
 * @author    Joel Schroeder
 */

public class RsfFactory {
  static public Rsf newRsf(rsfF.vector.Rsf src) {
    Rsf rsf = null;
    switch (src.getDimension()) {
    case 1:
      rsf = new Rsf1(src);
      break;
    case 2:
      rsf = new Rsf2(src);
      break;
    case 3:
      rsf = new Rsf3(src);
      break;
    case 4:
      rsf = new Rsf4(src);
      break;
    case 5:
      rsf = new Rsf5(src);
      break;
    case 6:
      rsf = new Rsf6(src);
      break;
    default:
      System.err.println("Error in RsfFactory: rsf.Rsf only supported up " +
			 "to 6 dimensions.\n");
      break;
    }
    return rsf;
  }

  public static Rsf newRsf(Rsf src) {
    Rsf newRsf = null;
    switch (src.getDimension()) {
    case 1:
      newRsf = new Rsf1((Rsf1) src);
      break;
    case 2:
      newRsf = new Rsf2((Rsf2) src);
      break;
    case 3:
      newRsf = new Rsf3((Rsf3) src);
      break;
    case 4:
      newRsf = new Rsf4((Rsf4) src);
      break;
    case 5:
      newRsf = new Rsf5((Rsf5) src);
      break;
    case 6:
      newRsf = new Rsf6((Rsf6) src);
      break;
    default:
      System.err.println("Error in RsfFactory: rsf.Rsf only supported up " +
			 "to 6 dimensions.\n");
      break;
    }
    return newRsf;
  }

  public static Rsf newRsf(InputStream istr) {
    return newRsf(new rsfF.vector.Rsf(istr));
  }
  
  public static Rsf newRsf(String fileName ) {
    try {
      return newRsf(new FileInputStream(new File(fileName)));
    }
    catch(Exception e ) {
      System.err.println(e);
      System.exit(-1);
    }
    return null;
  }
  
  public static Rsf newRsf(RsfSpace space    ) {
    Rsf rsf = null;
    switch (space.getDimension()) {
    case 1:
      rsf = new Rsf1(space);
      break;
    case 2:
      rsf = new Rsf2(space);
      break;
    case 3:
      rsf = new Rsf3(space);
      break;
    case 4:
      rsf = new Rsf4(space);
      break;
    case 5:
      rsf = new Rsf5(space);
      break;
    case 6: // = RsfSpace.maxDimensions
      rsf = new Rsf6(space);
      break;
    default:
      System.err.println("Error in RsfFactory.newRsf(): " +
			 "space has illegal dimension = " +
			 space.getDimension());
      break;
    }
    return rsf;
  }

  public static Rsf newRsf(Axis[]   axes     ) {
    return newRsf(new RsfSpace(axes));
  }
  public static Rsf newRsf(int []   nSamples ) {
    return newRsf(new RsfSpace(nSamples));
  }
  public static Rsf newRsf(int      n        ) {
    return new Rsf1(new RsfSpace(n));
  }
  /** n0 is the fastest index: array[n1][n0] (n1=columns, n0=rows) Bad! */  
  public static Rsf newRsf(int n0, int n1) {
    return new Rsf2(new RsfSpace(n0, n1));
  }
  /** n0 is the fastest index: array[n2][n1][n0]. Bad! See ../paper/README. */ 
  public static Rsf newRsf(int n0, int n1, int n2) {
    return new Rsf3(new RsfSpace(n0, n1, n2));
  }

  public static rsfF.vector.Rsf rsf2rsfF(Rsf src) {
    //rsfF.vector.Rsf rsf = new rsfF.vector.Rsf(src.space.getAxes());
    rsfF.vector.Rsf rsf = 
      new rsfF.vector.Rsf(src.space.getAxes(),src.space.getDataAxis());

    rsf.setTitle(src.getTitle()); 

    int[] i = new int[6];

    switch (src.space.getDimension()) {
    case 1:
      for (int j = 0; j < rsf.d.length; j++)
	rsf.d[j] = (((Rsf1) src).d[j]);
      break;
    case 2:
      Rsf2 src2 = (Rsf2) src;
      for   (int k = 0; k < src2.size1 ; k++) {
	int index = rsf.i(0,k);
	float[] data = src2.d[k];
	for (int j = 0; j   < data.length;   j++, index++)
	  rsf.d[index] = data[j];
      }
      break;
    case 3:
      Rsf3 src3 = (Rsf3) src;
      i[0] = 0;
      for     (i[2] = 0; i[2] < src3.size2; i[2]++)
	for   (i[1] = 0; i[1] < src3.size1; i[1]++) {
	  int index = rsf.i(i);
	  float[] data = src3.d[i[2]][i[1]];
	  for (int j = 0; j   < data.length;   j++, index++)
	    rsf.d[index] = data[j];
	}
      break;
    case 4:
      Rsf4 src4 = (Rsf4) src;
      i[0] = 0;
      for       (i[3] = 0; i[3] < src4.size3; i[3]++)
	for     (i[2] = 0; i[2] < src4.size2; i[2]++)
	  for   (i[1] = 0; i[1] < src4.size1; i[1]++) {
	    int index = rsf.i(i);
	    float[] data = src4.d[i[3]][i[2]][i[1]];
	    for (int j = 0; j   < data.length;   j++, index++)
	      rsf.d[index] = data[j];
	  }
      break;
    case 5:
      Rsf5 src5 = (Rsf5) src;
      i[0] = 0;
      for         (i[4] = 0; i[4] < src5.size4; i[4]++)
	for       (i[3] = 0; i[3] < src5.size3; i[3]++)
	  for     (i[2] = 0; i[2] < src5.size2; i[2]++)
	    for   (i[1] = 0; i[1] < src5.size1; i[1]++) {
	      int index = rsf.i(i);
	      float[] data = src5.d[i[4]][i[3]][i[2]][i[1]];
	      for (int j = 0; j   < data.length;   j++, index++)
		rsf.d[index] = data[j];
	    }
      break;
    case 6: // = RsfSpace.maxDimensions
      Rsf6 src6 = (Rsf6) src;
      i[0] = 0;
      for           (i[5] = 0; i[5] < src6.size5; i[5]++)
	for         (i[4] = 0; i[4] < src6.size4; i[4]++)
	  for       (i[3] = 0; i[3] < src6.size3; i[3]++)
	    for     (i[2] = 0; i[2] < src6.size2; i[2]++)
	      for   (i[1] = 0; i[1] < src6.size1; i[1]++) {
		int index = rsf.i(i);
		float[] data = src6.d[i[5]][i[4]][i[3]][i[2]][i[1]];
		for (int j = 0; j   < data.length;   j++, index++)
		  rsf.d[index] = data[j];
	      }
      break;
    default:
      System.err.println("Error in RsfFactory.rsf2rsfF(): " +
			 "space has illegal dimension = " +
			 src.space.getDimension());
      break;
    }
    return rsf;
  }
}
